#include "main.h"

void rendermon(struct wl_listener *listener, void *data) {
}

void cleanupmon(struct wl_listener *listener, void *data) {
}

void createmon(struct wl_listener *listener, void *data) {
	struct server *server = wl_container_of(listener, server, new_output);
	struct wlr_output *wlr_output = data;
	wlr_output_init_render(wlr_output, server->alloc, server->renderer);

	wlr_output_set_mode(wlr_output, wlr_output_preferred_mode(wlr_output));

	wlr_output_enable(wlr_output, 1);
	if (!wlr_output_commit(wlr_output))
		return;

	wlr_output_layout_add_auto(server->output_layout, wlr_output);
}

void server_init(struct server *server) {
	server->wl_display = wl_display_create();
	assert(server->wl_display);

	server->backend = wlr_backend_autocreate(server->wl_display);
	assert(server->backend);

	/* Create a renderer with the default implementation */
	server->renderer = wlr_renderer_autocreate(server->backend);
	assert(server->renderer);

	wlr_renderer_init_wl_display(server->renderer, server->wl_display);

	/* Create a default allocator */
	server->alloc = wlr_allocator_autocreate(server->backend, server->renderer);
	assert(server->alloc);

	wlr_data_device_manager_create(server->wl_display);
	wlr_export_dmabuf_manager_v1_create(server->wl_display);
	wlr_screencopy_manager_v1_create(server->wl_display);
	wlr_data_control_manager_v1_create(server->wl_display);
	wlr_gamma_control_manager_v1_create(server->wl_display);
	wlr_primary_selection_v1_device_manager_create(server->wl_display);

	/* Init monitors */
	server->new_output.notify = createmon;
	wl_signal_add(&server->backend->events.new_output, &server->new_output);
	wl_list_init(&server->outputs);

	const char *socket = wl_display_add_socket_auto(server->wl_display);
	assert(socket);

	printf("Running compositor on wayland display '%s'\n", socket);
	setenv("WAYLAND_DISPLAY", socket, 1);

	if (!wlr_backend_start(server->backend)) {
		wl_display_destroy(server->wl_display);
		die("wlr_backend_start", 1);
	}
}

int main(void) {
	if (!getenv("XDG_RUNTIME_DIR"))
		die("XDG_RUNTIME_DIR must be set", 0);

	int sig[] = {SIGCHLD, SIGINT, SIGTERM, SIGPIPE};
	for (size_t i = 0; i < sizeof(sig) / sizeof(sig[0]); i++)
			signal(sig[i], NULL);

	wlr_log_init(LOG_LEVEL, NULL);

	struct server server;
	server_init(&server);

	wl_display_run(server.wl_display);
	wlr_backend_destroy(server.backend);
	wl_display_destroy(server.wl_display);
	return 0;
}
